/*
 * Copyright 2008 by inovat, Dipl.-Ing. H. C. Kniß
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast;

//~ KLASSEN ===================================================================

/**
 * Klasse definietr die BASt-Konstanten.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class KonstantenBast {

    /**
     * Formate für BASt-Band Exporte/Importe.
     */
    public enum BAST_FORMAT {
        V2004(0, "V2004", "Export/Verkehr_Menge_2004", "Import/Verkehr_Menge_2004"),                      // BAStBestandsbandformat der Version 2004
        V2007(5, "V2007", "Export/Verkehr_Geschwindigkeit_2007", "Import/Verkehr_Geschwindigkeit_2007"),  // BASt-Bestandsbandformat für Geschwindigkeitsdaten in der Version 2007
        AXL_EF1(7, "AXL2009", "Export/Achslast_2009", "");                                                // EinzeldatenAchslast Bast Export

        public String _ftpExportOrdner;
        public String _ftpImportOrdner;
        int           _nummer;
        String        _text;

        //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==========

        BAST_FORMAT(int nummer, String text, String ftpExportOrdner, String ftpImportOrdner) {
            _nummer          = nummer;
            _text            = text;
            _ftpExportOrdner = ftpExportOrdner;
            _ftpImportOrdner = ftpImportOrdner;
        }

        //~ METHODEN ==========================================================

        /**
         * Ermittle den Format mit der angegebenen Nummer.
         *
         * @param nummer Nummer.
         *
         * @return Format mit der angegebenen Nummer oder null.
         */
        public static BAST_FORMAT ermittleFormat(int nummer) {
            for (BAST_FORMAT format : BAST_FORMAT.values()) {
                if (format._nummer == nummer) {
                    return format;
                }
            }

            return null;
        }

        /**
         * Ermittle den Format mit dem angegebenen Text.
         *
         * @param text Text.
         *
         * @return Format mit dem angegebenen Text oder null.
         */
        public static BAST_FORMAT ermittleFormat(String text) {
            for (BAST_FORMAT format : BAST_FORMAT.values()) {
                if (format._text.equals(text)) {
                    return format;
                }
            }

            return null;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 30.07.12
