/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.exporter.SendeBAStStatusDaten
 *
 * de.inovat.kex.bast.exporter.SendeBAStStatusDaten is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.exporter.SendeBAStStatusDaten is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.exporter.SendeBAStStatusDaten; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Köln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.dav.daf.main.config.*;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.common.NetzClientDavConnection;

import java.text.SimpleDateFormat;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet die Statusinformationen die an den Datenverteiler gesendet werden.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class SendeBAStStatusDaten implements ClientSenderInterface {

    /** Aspekt Zustand für die Zustandmeldung Export . */
    private static final String ASP_EXPORT_STATUS = "asp.bastExportStatus";

    /** Aspekt Zustand für die Zustandmeldung Import . */
    private static final String ASP_IMPORT_STATUS = "asp.bastImportStatus";

    /** Attributgruppe BASt-Status. */
    private static final String ATG_BAST_STATUS = "atg.bastStatus";

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    //~ FELDER ================================================================

    /**
     * Attribut welches die aktuelle Prozentzahl beinhaltet. Für Statusmeldungen ohne Fortschritt der Prozentangabe wird die aktuelle Prozentangabe beibehalten und in der Datenidentifikation
     * entsprechend gesetzt.
     */
    private String _prozent = "1";

    /** Datenbeschreibungsobjekt Export Status. */
    private DataDescription _datenbeschreibungExportImportStatus;

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** Systemobjekt für den BASt-Band Export. */
    private SystemObject _soBAStExportImportSteuerung;

    /** Typ des Systemobjekte für den BASt-Band Import/Export. */
    private String _typBAStBand;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erszeugt eine Instanz der Kalsse.
     *
     * @param typBAStBand Typ des Systemobjekte für den BASt-Band Import/Export.
     * @param isExport Parameter der angibt ob es sich um einen Exportstatus oder Importstatus handelt.
     */
    public SendeBAStStatusDaten(String typBAStBand, boolean isExport) {
        _dav         = NetzClientDavConnection.getInstance();
        _typBAStBand = typBAStBand;

        //
        anmeldenDaVSenderBAStStatus(isExport);
    }

    //~ METHODEN ==============================================================

    /**
     * Anmeldung beim Datenverteiler als Sender für den Status bzgl. Senden der BASt-Band Daten.
     *
     * @param isExport Parameter der angibt ob es sich um einen Exportstatus oder Importstatus handelt.
     */
    private void anmeldenDaVSenderBAStStatus(boolean isExport) {
        debug.info("Anmeldung der Daten zum Senden der Statusinformationen.");
        _soBAStExportImportSteuerung = _dav.getDataModel().getObject(_typBAStBand);

        AttributeGroup attributgruppeStatus = _dav.getDataModel().getAttributeGroup(ATG_BAST_STATUS);
        Aspect         aspektZustand;

        if (isExport) {
            aspektZustand = _dav.getDataModel().getAspect(ASP_EXPORT_STATUS);
        } else {
            aspektZustand = _dav.getDataModel().getAspect(ASP_IMPORT_STATUS);
        }

        _datenbeschreibungExportImportStatus = new DataDescription(attributgruppeStatus, aspektZustand);

        // Anmelden zum Senden der Daten
        try {
            _dav.subscribeSender(this, _soBAStExportImportSteuerung, _datenbeschreibungExportImportStatus, SenderRole.source());
        }
        catch (OneSubscriptionPerSendData e) {
            debug.error("Es konnte keine Verbindung zum Datenverteiler hergestellt werden. ", e);
        }
    }

    public void dataRequest(SystemObject systemObject, DataDescription dataDescription, byte b) {}

    //~ GET METHODEN ==========================================================

    public boolean isRequestSupported(SystemObject systemObject, DataDescription dataDescription) {
        return false;
    }

    //~ METHODEN ==============================================================

    /**
     * Senden des aktuellen Status.
     *
     * @param prozent       Prozentangabe des aktuellen Exportvorganges.
     * @param statusmeldung Statusmeldung zum aktuellen Exportvorgang.
     *
     * @throws de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed Exception.
     *          .
     */
    public void sendeBAStStatusDaten(String prozent, String statusmeldung) throws SendSubscriptionNotConfirmed {
        if (prozent.equals("") && prozent.equals(_prozent)) {

            // nothing to do ...
        } else {

            // Zeitstempel
            long             zeitstempel = System.currentTimeMillis();
            SimpleDateFormat sdf         = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");
            String           time        = sdf.format(zeitstempel);

            // Datensatz erstellen
            AttributeGroup atg                        = _dav.getDataModel().getAttributeGroup(ATG_BAST_STATUS);
            Data           datensatzExportStatus      = _dav.createData(atg);
            Data.TextValue bastBandFortschrittProzent = datensatzExportStatus.getTextValue("BAStBandFortschrittProzent");

            // Wenn ein leerer String als Prozentänderung übergeben wird, soll sich an der Prozentzahl Progressbar nichts ändern,
            // dann handelt es sich um eine Information für den Benutzer.
            if (prozent.equals("")) {
                bastBandFortschrittProzent.setText(_prozent);
            } else {
                _prozent = prozent;
                bastBandFortschrittProzent.setText(prozent);
            }

            // Tritt nie auf!
            if (Integer.parseInt(_prozent) < 1) {
                _prozent = "1";
                bastBandFortschrittProzent.setText(_prozent);
            }

            Data.TextValue statusMeldung = datensatzExportStatus.getTextValue("StatusMeldung");

            if (!statusmeldung.equals("")) {
                statusMeldung.setText(time + ": " + statusmeldung + System.getProperty("line.separator"));
            } else {
                statusMeldung.setText("");
            }

            // Telegramm erstellen
            ResultData datenExportStatusResultData = new ResultData(_soBAStExportImportSteuerung, _datenbeschreibungExportImportStatus, zeitstempel, datensatzExportStatus);

            // Versenden
            try {
                _dav.sendData(datenExportStatusResultData);
            }
            catch (DataNotSubscribedException dataNotSubscribed) {
                debug.error("Die Daten konnten nicht versendet werden." + dataNotSubscribed);
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.08.12
