/**
 * Copyright (c) 2004-2007 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;


/*
 * History:
 * 2007-09-27 [KS]  Code-Review, Logger, JavaDoc
 *
 * Revision 1.5  2007/11/26 12:10:19  Trier
 * <Kein Kommentar eingegeben>
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.inovat.kex.bast.common.*;

/**
 * <p>Kapselt den Datenmodell-Typ KRI ("typ.kri").</p>
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: KRI.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class KRI extends ConfigurationObjectWrapper {

//  private static final de.inovat.kex.bast.funclib.logger.Logger logger =
//    de.inovat.kex.bast.funclib.logger.LoggerFactory.getLoggerFactory().getLogger(KRI.class);

  private static WeakHashMap<String,KRI> __Cache = new WeakHashMap<String,KRI>();
  private static ConfigurationObjectType __ConfigurationObjectType;
  private static ConfigurationObjectType __ConfigurationObjectTypeOfSteuerModul;
  private static AttributeGroup __AtgGeraet;

  private String _Bezeichnung;
  private int _KnotenNummer;
  private String _UmsetzungsModul;
  private ArrayList<KriBus> _BusList;

  /**
   * <p>Liefert die Konfigurationsangabe zum Attribut "Bezeichnung".<p>
   * @return Bezeichnung
   */
  public String getBezeichnung() {
    return _Bezeichnung;
  }

  /**
   * <p>Liefert die Konfigurationsangabe zum Attribut Knotennummer (OSI7
   * Adresse des Geräts).</p>
   *
   * @return Knotennummer
   */
  public int getKnotenNummer() {
    return _KnotenNummer;
  }

  /**
   * <p>Gibt das für die Umsetzung des Protokolls (Lesen und Schreiben)
   * verwendete Modul an diesem Knoten zurück.</p>
   *
   * @return Klassennamen des UmsetzungsModuls.
   */
  public String getUmsetzungsModul() {
    return _UmsetzungsModul;
  }

  /**
   * <p>Erzeugt das Wrapper-Object für das Konfigurationsobjekt mit der
   * angegebenen Pid.</p>
   *
   * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
   *
   * @param pid String
   * @throws DataModellMismatchException Exception.
   * @throws DataModellObjectNotFoundException Exception.
   */
  private KRI(String pid) throws DataModellMismatchException, DataModellObjectNotFoundException {
    super(pid);
  }

  /**
   * <p>Erzeugt das Wrapper-Object für das angegebene Konfigurationsobjekt.</p>
   *
   *
   * @param confObj ConfigurationObject
   * @throws DataModellMismatchException Exception.
   */
  private KRI(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }

  /**
   * <p>Gibt das Wrapper-Objekt zum Konfigurationsobjekt mit der angegebenen
   * Pid zurück <code>null</code> wenn es keinen KRI mit der Pid gibt.</p>
   *
   * @param pid String
   * @throws DataModellMismatchException Exception.
   * @return SteuerModul
   */
  public static KRI getInstance(String pid) throws DataModellMismatchException {
    KRI kri = __Cache.get(pid);
    if (kri==null) {
      Object object = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((object!=null) && (object instanceof ConfigurationObject)) {
        kri = getInstanceIntern((ConfigurationObject)object);
      }
    }
    return kri;
  }

  /**
   * <p>Liefert das Wrapper-Objekt zum angegebenen Konfigurationsobjekt.</p>
   *
   * @param confObj ConfigurationObject
   * @return SteuerModul
   * @throws DataModellMismatchException Exception.
   */
  static public KRI getInstance(ConfigurationObject confObj) throws DataModellMismatchException {
    KRI kri = __Cache.get(confObj.getPid());
    if (kri == null) {
      kri =  getInstanceIntern(confObj);
    }
    return kri;
  }

  static synchronized private KRI getInstanceIntern(ConfigurationObject confObj) throws DataModellMismatchException {
    KRI kri = __Cache.get(confObj.getPid());
    if (kri == null) {
      if (__ConfigurationObjectType == null) {
        init();
      }
      kri = new KRI(confObj);
    }
    return kri;
  }

  /**
   * <p>Prüft und Initialisiert das Wrapper-Objekt mit den Konfigurationsdaten
   * des angegebenen Konfigurationsobjekts.</p>
   *
   * @param confObj ConfigurationObject
   * @throws DataModellMismatchException Exception.
   */
  protected void initData(ConfigurationObject confObj) throws DataModellMismatchException {
    Data oData = confObj.getConfigurationData(__AtgGeraet);
    if (oData != null) {
      _Bezeichnung = oData.getTextValue("Bezeichnung").getText();
      _KnotenNummer = oData.getUnscaledValue("KnotenNummer").intValue();
      _UmsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
    }
    ObjectSet os = confObj.getObjectSet("AnschlussPunkteGerät");
    if (os != null) {
      List<SystemObject> list = os.getElements();
      if ((list != null) && (list.size() > 0)) {
        int iListSize = list.size();
        _BusList = new ArrayList<KriBus>(iListSize);
        for (int i = 0; i < iListSize; i++) {
          ConfigurationObject busObj = (ConfigurationObject) list.get(i);
          KriBus bus = new KriBus(busObj.getPid());
          _BusList.add(bus);
          List<SystemObject> kpList = ConfigurationObjectWrapper.getKommunikationPartner(busObj);
          if ((kpList != null) && (kpList.size() > 0)) {
            for (int kpItemNr = 0; kpItemNr < kpList.size(); kpItemNr++) {
              ConfigurationObject smObj = (ConfigurationObject) kpList.get(kpItemNr);
              if (smObj.isOfType(__ConfigurationObjectTypeOfSteuerModul)) {
                bus.addSteuerModul(SteuerModul.getInstance(smObj));
              }
            }
          }
        }
      }
    }
    __Cache.put(this.getPID(), this);
  }

  /**
   * <p>Liefert die am KRI konfigurierten Busse.</p>
   * @return KriBus[]
   */
  public KriBus[] getBuse() {
    if ((_BusList!=null) && (_BusList.size()>0)) {
      return _BusList.toArray(new KriBus[_BusList.size()]);
    }
    else {
      return null;
    }
  }

  /**
    * <p>Gibt ein Array aller Wrapper-Objekte des zurück, die vom Typ "typ.kri"
    * sind.</p>
    *
    * @return Unterzentrale[]
   * @throws DataModellMismatchException Exception.
   */
  public static KRI[] getAll() throws DataModellMismatchException {
     if (__ConfigurationObjectType == null) {
       init();
     }
     KRI[] retValue;
     if (__ConfigurationObjectType!=null) {
       List<SystemObject> list = __ConfigurationObjectType.getElements();
       retValue = new KRI[list.size()];
       for (int i = 0; i < list.size(); i++) {
         retValue[i] = KRI.getInstanceIntern((ConfigurationObject) list.get(i));
       }
     }
     else {
       retValue = new KRI[0];
     }
     return retValue;
  }


  /**
   * <p>Liefert den Typ der Konfigurationsobjekte "typ.kri" zurück.</p>
   * @return ConfigurationObjectType
   */
  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  /**
   * <p>Liefert den Namen des Konfigurationsobjekts für die Anzeige im Baum.</p>
   * @return String
   */
  public String getTreeString() {
    return getName();
  }

  /**
   * <p>Initialisiert allgemeine Konfigurationsdaten f&uuml;r den typ.kri des Datenmodells.</p>
   */
  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.kri");
      __ConfigurationObjectTypeOfSteuerModul = (ConfigurationObjectType)model.getType("typ.steuerModul");
      __AtgGeraet = model.getAttributeGroup("atg.gerät");
      List<SystemObject> list = __ConfigurationObjectType.getObjects();
      model.getConfigurationData(list, __AtgGeraet);
    }
  }

}
