/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.inovat.kex.bast.common.objectwrapper;


/*
 * Revision 1.3  2007/11/26 12:10:20  Trier
 * <Kein Kommentar eingegeben>
 */
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.kex.bast.common.*;

/**
 * kapselt den DatenModellTyp straße
 * 
 * @author LOGOS GmbH Rostock (www.logos-ib.de)
 * @author Ralf Trier
 * @version $Id: Strasse.java 442 2008-11-10 06:59:32Z Schawaller $
 */
public class Strasse extends ConfigurationObjectWrapper {

  /** DebugLogger fuer Debug-Ausgaben. */
  private static Debug debug = Debug.getLogger();

  private static WeakHashMap<String, Strasse> __Cache = new WeakHashMap<String, Strasse>();
  private static ConfigurationObjectType __ConfigurationObjectType;

  private String _Typ;
  private long _Nummer;
  private String _Zusatz;

  private static AttributeGroup __AtgStrasse;

  public String getTyp() {
    return _Typ;
  }
  public long getNummer() {
    return _Nummer;
  }
  public String getZusatz() {
    return _Zusatz;
  }


  private Strasse(String pid) throws DataModellObjectNotFoundException, DataModellMismatchException {
    super(pid);
  }

  private Strasse(ConfigurationObject confObj) throws DataModellMismatchException {
    super(confObj);
  }

  protected void initData(ConfigurationObject confObj) throws DataModellMismatchException {

    Data oData = confObj.getConfigurationData(__AtgStrasse);
    if (oData != null) {
      if (oData.getUnscaledValue("Nummer").isNumber()) {
        try {
          _Nummer = oData.getUnscaledValue("Nummer").longValue();
        }
        catch (Exception e) {
          e.printStackTrace();
        }
      }
      if (oData.getUnscaledValue("Typ").isState()) {
        _Typ = oData.getUnscaledValue("Typ").getState().getName();
      }
      _Zusatz = oData.getTextValue("Zusatz").getText();
    }
    else {
      debug.warning("Keine atg.straße für Object: " + getPID() + " vorhanden." );
    }

    __Cache.put(this.getPID(), this);

  }

  /**
   * gibt die Instance der Strasse mit der PID zurück. Es wird eine
   * DataModellObjectNotFoundException geworfen, wenn die PID nicht vorhanden
   * ist oder das zur PID gehörende Objekt nicht vom typ straße ist.
   *
   * @param confObj ConfigurationObject
   * @throws DataModellMismatchException Exception.
   * @return Strasse
   */
  public static Strasse getInstance(ConfigurationObject confObj) throws DataModellMismatchException {
    Strasse strasse = __Cache.get(confObj.getPid());
    if (strasse == null) {
      strasse = Strasse.getInstanceIntern(confObj);
    }
    return strasse;
  }

  public static Strasse getInstance(String pid) throws DataModellMismatchException {
    Strasse strasse = __Cache.get(pid);
    if (strasse == null) {
      Object o = NetzClientDavConnection.getInstance().getDataModel().getObject(pid);
      if ((o!=null) && (o instanceof ConfigurationObject)) {
        strasse = Strasse.getInstanceIntern((ConfigurationObject)o);
      }
    }
    return strasse;
  }


  private static synchronized Strasse getInstanceIntern(ConfigurationObject confObj) throws DataModellMismatchException {
    Strasse strasse = __Cache.get(confObj.getPid());
    if (strasse == null) {
      if (__ConfigurationObjectType == null) {
        Strasse.init();
      }
      strasse = new Strasse(confObj);
    }
    return strasse;
  }

  /**
   * gibt die DatenModellObjekte vom typ straße zurück
   *
   * @throws DataModellMismatchException Exception.
   * @return Strasse[]
   */
  public static Strasse[] getAll() throws DataModellMismatchException {
    DataModel model = NetzClientDavConnection.getInstance().getDataModel();
    List<SystemObject> list = model.getType("typ.straße").getElements();
    Strasse[] retValue = new Strasse[list.size()];
    for (int i = 0; i < list.size(); i++) {
      retValue[i] = Strasse.getInstanceIntern( (ConfigurationObject) list.get(i));
    }
    return retValue;
  }

  ConfigurationObjectType getType() {
    return __ConfigurationObjectType;
  }

  public String getTreeString() {
    return getName();
  }

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("Strasse -- PID: \"").append(getPID());
    sb.append("\"  Name: \"").append(getName());
    sb.append("\"  Nummer: \"").append(_Nummer);
    sb.append("\"  Typ: \"").append(_Typ);
    sb.append("\"  Zusatz: \"").append(_Zusatz);
    return sb.toString();
  }


  private static synchronized void init() {
    if (__ConfigurationObjectType==null) {
      DataModel model = NetzClientDavConnection.getInstance().getDataModel();
      __ConfigurationObjectType = (ConfigurationObjectType)model.getType("typ.straße");
      __AtgStrasse = model.getAttributeGroup("atg.straße");
      List<SystemObject> list = __ConfigurationObjectType.getObjects();
      model.getConfigurationData(list, __AtgStrasse);
    }
  }

}
