/*
 * Copyright (c) 2010 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.kex.bast.exporter.DataChannelAxlBAStEF1
 *
 * de.inovat.kex.bast.exporter.DataChannelAxlBAStEF1 is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.kex.bast.exporter.DataChannelAxlBAStEF1 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.kex.bast.exporter.DataChannelAxlBAStEF1; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Köln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.kex.bast.exporter;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.sys.funclib.debug.Debug;

import de.inovat.kex.bast.common.ChannelException;
import de.inovat.kex.bast.common.objectwrapper.AuswerteQuerschnitt;
import de.inovat.kex.bast.common.objectwrapper.Fahrstreifen;
import de.inovat.kex.bast.common.objectwrapper.LangZeitZaehlStelle;
import de.inovat.kex.bast.SendeBAStStatusDaten;

//~ JDK IMPORTE ===============================================================

import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Das Format Einzelfahrzeugdaten - EF1 dient zur Übermittlung der Daten von Achslastmessstellen an die BASt, sofern die
 * Daten nicht direkt von der Messstelle an die BASt weitergeleitet werden.
 * <p>
 * Die eingehenden Daten von der Achslastmesstele werden in dem Typ typ.deAxl, der Attributgrupe
 * atg.tlsEinzelErgebnisMeldungAchslastenUndKfzGesamtGewichteVersion3  und dem Aspekt asp.tlsAntwort an den
 * Datenverteiler publiziert.  Dementsprechend werden genau diese Datensätze aus dem Archiv ausgelesen und in die
 * Exportdatei geschrieben. Folgende Schritte werden dabei vollzogen:
 * <ul>
 * <li> 1. Bestimmung der DE-Kanäle zu der
 * übergebenen Langzeitzählstelle. Fehlermeldung, wenn es sich nicht um eine Achslasterfassende Langzeitzählstelle
 * Handelt. </li> <li> 2. Auslesen der Daten aus dem Archiv (zu Prüfen ob es zu Performanceeinbußen kommt, wenn die
 * Tagesdatensätze ausgelesen werden und nicht der Monatsdatensatz...) </li> <li> 3. Bestimmung des Filenamens (pro Tag,
 * pro Zählstelle wird ein Dateiname erzeugt) </li> <li> 4. Erzeugung der Datei mit bestimmten Dateinamen auf dem
 * FTP-Server. </li> <li> 5. Schreiben der Stundendatensätze in die Datei, wobei eine Konvertierung in Binärformat
 * durchgeführt wird. </li> <li> 6. Bestimmung der Länge des Austauschformates, schreiben dieser Länge an Position Byte
 * 1. </li>
 * </ul>
 * <p>
 * Für jeden Tag und jede Messstelle wird eine Datei angelegt, in der die Achslastmessstelldaten im Binärformat
 * enthalten sind.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dagmar Rostek (DR)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class DataChannelAxlBAStEF1 extends AbstractDataChannelBASt {

    /**
     * Konstante zum Erzeugen ein Channels f&uuml;r die GEGEN-Richtung
     */
    public final static int GEGEN_RICHTUNG = 11;

    /**
     * Konstante zum Erzeugen ein Channels f&uuml;r die IN-Richtung
     */
    public final static int IN_RICHTUNG = 10;

    /** Länge für einen zu exportierenden Datensatz für deLve. */
    private static final int LENGTH_LVE = 16;

    /** DebugLogger fuer Debug-Ausgaben. */
    private static Debug debug = Debug.getLogger();

    /** Minimallänge für einen zu exportierenden Datensatz. */
    private static final int MINLENGTH = 21;

    //~ FELDER ================================================================

    /** Array mit der binären Repräsentation des zu exportierenden Datensatzes. */
    private int[] _arIntValues;

    /** Attribut Auswertequerschnitt, repräsentiert ein Objekt der Klasse. */
    private AuswerteQuerschnitt _AuswerteQuerschnitt;

    /** Kopie des Arrays zur Anpassung des Binärarrays entsprechend der zu exportierenden Datensätze. */
    private int[] _copyArIntValues;

    /**
     * ClientDavInteface.
     */
    private ClientDavInterface _dav;

    /** Attribut Fernziel. */
    private String _Fernziel;

    /** Array der Fahrstreifen. */
    private Fahrstreifen[] _FSArray;

    /** DatenerfassungsArray. */
    private Fahrstreifen[] _FSMitDatenerfassungArray;

    /** Richtungsangabe. */
    private int _Richtung;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public DataChannelAxlBAStEF1(int inRichtung, ClientDavInterface dav, SendeBAStStatusDaten bastStatusDaten) {
        super(bastStatusDaten);
        _dav      = dav;
        _Richtung = inRichtung;
    }

    //~ METHODEN ==============================================================

    /**
     * Anpassung des Wertes. Wert aus Dav wird in ef1-Format-Wert abgebildet.
     *
     * @param wert                Wert aus DaV.
     * @param anzByte             anzahl der Bytes für den Wert im ef1-Format.
     * @param nichtErmittelbarDav Nicht ermittelbar für DaV-Wert.
     *
     * @return Wert im ef1-Format.
     */
    private int anpassenWert(int wert, int anzByte, int nichtErmittelbarDav) {

        // Wert ist nicht ermittelbar:
        // 1 Byte -- 255
        // 2 Byte -- 65535
        // 3 Byte -- 16777215
        int nichtErmittelbar = (int) (Math.pow(2, anzByte * 8)) - 1;
        int maxWert          = nichtErmittelbar - 1;

        //
        if (wert == nichtErmittelbarDav) {
            wert = nichtErmittelbar;
        }

        //
        else if (wert > nichtErmittelbarDav) {
            wert = maxWert;
        }

        //
        else if (wert > nichtErmittelbar) {
            wert = maxWert;
        }

        return wert;
    }

    /**
     * Die Methode ermittelt die Fahrstreifenkennung für den übergebenen Datensatz. Die Fahrstreifenkennung ist gleich
     * dem DEKanal.
     *
     * @param rsArrayAtTime Datensatz für den die Fahrstreifenkennung ermittelt wird.
     *
     * @return DEKanal.
     */
    private int bestimmeFahrstreifenkennung(Dataset[] rsArrayAtTime) {
        List<SystemObject> listSO = new ArrayList<SystemObject>();

        try {
            Dataset        dataSet          = rsArrayAtTime[0];
            AttributeGroup atgDE            = _dav.getDataModel().getAttributeGroup("atg.de");
            Aspect         aspEigenschaften = _dav.getDataModel().getAspect("asp.eigenschaften");
            SystemObject   soDeAxl          = _dav.getDataModel().getObject(dataSet.getObject().getPid());

            listSO.add(soDeAxl);

            Data[] konfigurierendeDaten = _dav.getDataModel().getConfigurationData(listSO, atgDE, aspEigenschaften);
            Data   data                 = konfigurierendeDaten[0];

            return data.getUnscaledValue("DEKanal").intValue();
        }
        catch (Exception e) {
            debug.error("Der DEKanal kann nicht bestimmt werden. Bitte wählen Sie eine Achslastmessstelle für den gewählten Export.");
        }

        return -1;
    }

    /**
     * Die Methode bestimmt den Wert für die 4 höchsten Bit. 5..6 = 0 7: 0 keine Ueberschreitung/1 Ueberschreitung
     *
     * @param high4BitsIntValue der Integerwert, für den der Wert bestimmt wird.
     * @param ueberschreitung   Bit das angibt ob eine Ueberschreitung vor liegt oder nicht.
     *
     * @return byte mit entsprechend angepasstem Bit an Position 7.
     */
    private int bestimmeHigh4Bits(int high4BitsIntValue, int ueberschreitung) {
        if (ueberschreitung != 0) {
            return setBit(high4BitsIntValue, 7);
        } else {
            return high4BitsIntValue;
        }
    }

    /**
     * Die Methode bestimmt für den uebergebenen Integer Wert die zugehörigen 4 niedrigsten Bit. Zum Beispiel wird das
     * Byte bestimmt, welches den Zustand des Attributes Ueberladung repräsentiert. Mit diesem Byte wird eine
     * Ueberschreitung des zulaessigen Gesamtgewichts oder Achsgewichts gekennzeichnet. Bit 0..4 ist wie folgt
     * umzusetzten: <li> 0: keine Ueberladung </li> <li> 1: Ueberschreitung des Gesamtgewichts </li> <li> 2:
     * Ueberschreitung mindestens eines Einzel-, Doppel, oder Dreifachachsgewichts des Fahrzeugs </li> <li> 3:
     * Ueberschreitung des Gesamtgewichts und mindestens eines Einzel-, Doppel, oder Dreifachgewichts des Fahrzeugs
     * </li>
     *
     * @param lower4BitsIntValue Ueberladung.
     *
     * @return Inhalt Byte Ueberladung.
     */
    private int bestimmeLower4Bits(int lower4BitsIntValue) {
        return (lower4BitsIntValue & 0xF);
    }

    /**
     * Die Methode bestimmt die Telegrammdaten. <li> Fahrzeugtyp </li> <li> Fahrzeuglänge low Byte</li> <li>
     * Fahrzeuglänge high Byte </li>
     *
     * @param data Daten anhand die zu extrahierenden Werte bestimmt werden.
     *
     * @return Anzahl der Dynamischen Telegramm Parameter "ÜberlastungAchse".
     */
    private int bestimmeTelegrammDatenFest(Data data) {

        // ---------------------------------------------------------------
        _arIntValues[10] = data.getUnscaledValue("FzTyp").intValue();

        // ---------------------------------------------------------------
        // Fahrzeuglänge im ef1-Format soll in Zentimeter übermittelt werden
        // (im DaV ist der Wert in Dezimetern gespeichert):
        int laenge = data.getUnscaledValue("FzLänge").intValue() * 10;

        laenge           = anpassenWert(laenge, 2, 65535 * 10);
        _arIntValues[11] = getLowByte(laenge);
        _arIntValues[12] = getHighByte(laenge);

        // ---------------------------------------------------------------
        int vFz = data.getUnscaledValue("vFz").intValue();

        vFz              = anpassenWert(vFz, 1, 255);
        _arIntValues[13] = vFz;

        // ---------------------------------------------------------------
        // Abstand im ef1-Format soll in Zentimeter übermittelt werden
        // (im DaV ist der Wert in Dezimetern gespeichert):
        int abstand = data.getUnscaledValue("FzAbstand").intValue() * 10;

        abstand          = anpassenWert(abstand, 2, 65535 * 10);
        _arIntValues[14] = getLowByte(abstand);
        _arIntValues[15] = getHighByte(abstand);

        // ---------------------------------------------------------------
        int gewicht = data.getUnscaledValue("FzGesamtGewicht").intValue();

        gewicht          = anpassenWert(gewicht, 3, 16777215);
        _arIntValues[16] = getLowByte(gewicht);
        _arIntValues[17] = getHighByte(gewicht);
        _arIntValues[18] = getExtraHighByte(gewicht);

        // ---------------------------------------------------------------
        _arIntValues[19] = bestimmeLower4Bits(data.getUnscaledValue("FzÜberladung").intValue());

        // ---------------------------------------------------------------
        Data.Array arUeberlastungAchse = data.getArray("ÜberlastungAchse");

        _arIntValues[20] = arUeberlastungAchse.getLength();

        return arUeberlastungAchse.getLength();
    }

    /**
     * Die Methode bestimmt die Telegrammdaten für deLve. <li> Fahrzeugtyp </li> <li> Fahrzeuglänge low Byte</li> <li>
     * Fahrzeuglänge high Byte </li>
     *
     * @param data Daten anhand die zu extrahierenden Werte bestimmt werden.
     */
    private void bestimmeTelegrammDatenFestLve(Data data) {

        /*
         *  Abbildung von Werten DaV --> ef1:
         * NAME                                            DaV ef1
         * Pkw-Gruppe                                       1  201
         * Pkw mit Anhänger                                 2  202
         * Lkw                                              3  203
         * Lkw mit Anhänger und Sattelkraftfahrzeuge        4  204
         * Busse                                            5  205
         * nicht klassifizierbare Kfz                       6  206
         * Pkw                                              7  207
         * Lkw mit Anhänger                                 8  208
         * Sattelkraftfahrzeuge                             9  209
         * Krad                                            10  210
         * Lieferwagen                                     11  211
         * Pkw-Ähnliche                                    32  201
         * Lkw-Ähnliche                                    33  203
         * Pkw-Ähnliche und Lkw-Ähnliche (alle Fahrzeuge)  64  206
         * ???                                             xx  206
         */
        int fzKlasse = data.getUnscaledValue("FZKlasse").intValue();

        if ((fzKlasse >= 1) && (fzKlasse <= 11)) {
            fzKlasse = fzKlasse + 200;
        } else if (fzKlasse == 32) {
            fzKlasse = 201;
        } else if (fzKlasse == 33) {
            fzKlasse = 203;
        } else {
            fzKlasse = 206;
        }

        _arIntValues[10] = fzKlasse;

        // ---------------------------------------------------------------
        // Fahrzeuglänge im ef1-Format soll in Zentimeter übermittelt werden
        // (im DaV ist der Wert in Dezimetern gespeichert):
        int laenge = data.getUnscaledValue("FZLänge").intValue() * 10;

        laenge           = anpassenWert(laenge, 2, 255 * 10);
        _arIntValues[11] = getLowByte(laenge);
        _arIntValues[12] = getHighByte(laenge);

        // ---------------------------------------------------------------
        int vKmH = data.getUnscaledValue("vKfz").intValue();

        vKmH             = anpassenWert(vKmH, 1, 32768);
        _arIntValues[13] = vKmH;

        // ---------------------------------------------------------------
        // Abstand wird berechnet aus Intervall in Sekunden und Geschwindigkeit in km/h:
        // Abstand in km = G * (I / (60*60))
        // In ef1-Format wird abstand in cm angegeben.
        double intervalSek = data.getUnscaledValue("tNetto").doubleValue() / 100;
        int    abstand;

        // Falls Intervall oder Geschwindigkeit nicht ermittelbar sind,
        // wird Abstand auch nicht ermittelbar:
        if ((Math.round(intervalSek) == 655) || (vKmH == 255)) {
            abstand = 65535;
        }

        // berechne den Abstand:
        else {
            double abstandCm = intervalSek * vKmH * 1000 * 100 / 60 / 60;

            abstand = (int) Math.round(abstandCm);
            abstand = anpassenWert(abstand, 2, 65535);
        }

        _arIntValues[14] = getLowByte(abstand);
        _arIntValues[15] = getHighByte(abstand);
    }

    /**
     * Die Methode bestimmt die variablen UeberlastungsAchsendaten.
     *
     * @param index Index gibt an, ab wo das Array mit Daten gefüllt wird.
     * @param data  Dataset mit den aktuellen Werten für die ein Export durchgeführt wird.
     */
    private void bestimmeTelegrammDatenvariabel(int index, Data data) {

        // ---------------------------------------------------------------
        // Der Startindex ist abhängig von der Anzahl der betroffenen Achsen.
        int startIndex = (index * 5) + 21;

        // ---------------------------------------------------------------
        Data.Array arUeberlastungAchse = data.getArray("ÜberlastungAchse");
        Data       ueberlastungAchse   = arUeberlastungAchse.getItem(index);

        // ---------------------------------------------------------------
        // Auslesen der Daten aus dem Telegramm.
        // Eine der drei3 folgenden Variablen kann Wert 0 (keine Überschreitung) oder 1 (Überschreitung) haben.
        // Andere beide haben Wert 3 (nicht betroffen).
        // Somit kann man den Achsentyp ermitteln.
        int ueberlastEinzelAchse   = ueberlastungAchse.getUnscaledValue("ÜberlastEinzelachse").intValue();
        int ueberlastDoppelAchse   = ueberlastungAchse.getUnscaledValue("ÜberlastDoppelachse").intValue();
        int ueberlastDreifachAchse = ueberlastungAchse.getUnscaledValue("ÜberlastDreifachachse").intValue();

        // Bestimmung welcher der Achsen betroffen ist & Belegung des Wertes ob es sich um eine
        // Ueberschreitung handelt oder nicht.
        int betroffeneAchse = 0;
        int ueberschreitung = 0;

        if (ueberlastEinzelAchse != 3) {
            betroffeneAchse = 1;
            ueberschreitung = ueberlastEinzelAchse;
        } else if (ueberlastDoppelAchse != 3) {
            betroffeneAchse = 2;
            ueberschreitung = ueberlastDoppelAchse;
        } else if (ueberlastDreifachAchse != 3) {
            betroffeneAchse = 3;
            ueberschreitung = ueberlastDreifachAchse;
        }

        // ---------------------------------------------------------------
        // Belegen des Bytearrays mit den bestimmten Byte Werten.
        _arIntValues[startIndex] = bestimmeHigh4Bits(bestimmeLower4Bits(betroffeneAchse), ueberschreitung);

        // ---------------------------------------------------------------
        int achslast;

        if (BAStExportSteuerung.getVersionAxlEinzeldaten() == 2) {
            int radLastLinks  = ueberlastungAchse.getUnscaledValue("RadlastLinks").intValue();
            int radLastRechts = ueberlastungAchse.getUnscaledValue("RadlastRechts").intValue();

            // Wenn beide Werte ungültig, dann Gesamtwert auch ungültig...
            if ((radLastLinks == 255) && (radLastRechts == 255)) {
                achslast = 65535;
            } else {

                // ... sonst testen, ob einer ungültig ist und diesen dann auf 0 setzten...
                if (radLastLinks == 255) {
                    radLastLinks = 0;
                }

                if (radLastRechts == 255) {
                    radLastRechts = 0;
                }

                // ... und dann die Achslast berechnen
                achslast = (radLastLinks + radLastRechts) * 100;  // Skalierung von 0,1t auf kg !!!
            }
        } else {
            achslast = ueberlastungAchse.getUnscaledValue("Achslast").intValue();
        }

        achslast                     = anpassenWert(achslast, 2, 65535);
        _arIntValues[startIndex + 1] = getLowByte(achslast);
        _arIntValues[startIndex + 2] = getHighByte(achslast);

        // ---------------------------------------------------------------
        int achsenabstand = ueberlastungAchse.getUnscaledValue("Achsabstand").intValue();

        achsenabstand                = anpassenWert(achsenabstand, 2, 65535);
        _arIntValues[startIndex + 3] = getLowByte(achsenabstand);
        _arIntValues[startIndex + 4] = getHighByte(achsenabstand);
    }

    /**
     * Die Methode bestimmt die Zeitbereichsangabe für den Datensatz anhand des Calendar Objektes.
     *
     * @param calendar Calendarobjekt.
     */
    private void bestimmeZeitangabe(Calendar calendar) {
        int    jahr          = calendar.get(Calendar.YEAR);
        String jahrAlsString = jahr + "";
        int    jahresZahl    = Integer.parseInt(jahrAlsString.subSequence(2, 4).toString());

        _arIntValues[3] = jahresZahl;
        _arIntValues[4] = calendar.get(Calendar.MONTH) + 1;  // Monate im Calendarobjekt sind von 0..11  durchnumeriert
        _arIntValues[5] = calendar.get(Calendar.DAY_OF_MONTH);
        _arIntValues[6] = calendar.get(Calendar.HOUR_OF_DAY);
        _arIntValues[7] = calendar.get(Calendar.MINUTE);
        _arIntValues[8] = calendar.get(Calendar.SECOND);
        _arIntValues[9] = calendar.get(Calendar.MILLISECOND) / 10;
    }

    /**
     * Die Methode bestimmt einen zweistelligen Monats oder Tageswert.
     *
     * @param value der Tageswert, oder Monatswert, der bzgl. Zweistelligkeit geprüft und ggf. zweistellig transformiert
     *              wird.
     *
     * @return zweistelliger Wert des übergebenem Wertes.
     */
    private String bestimmeZweiStelligeDatumsangaben(String value) {
        if (value.length() < 2) {
            return String.format("0%s", value);
        } else {
            return value;
        }
    }

    @Override
    protected String constructFilename(LangZeitZaehlStelle zaehlStelle, int year, int month) {
        return null;  // To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
    protected String constructFilename(LangZeitZaehlStelle zaehlStelle, int year, int month, int day) {
        StringBuilder sb         = new StringBuilder();
        String        jahresZahl = String.valueOf(year).substring(2);

        sb.append(zaehlStelle.getBastNummerAsString());
        sb.append("_");
        sb.append(bestimmeZweiStelligeDatumsangaben(String.valueOf(day)));
        sb.append(bestimmeZweiStelligeDatumsangaben(String.valueOf(month)));
        sb.append(jahresZahl);
        sb.append(".ef1");

        return sb.toString();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Die Methode bestimmt ExtraHighByte.
     *
     * @param value Wert für den das ExtraHighByte bestimmt wird.
     *
     * @return ExtraHighByte Wert.
     */
    private int getExtraHighByte(int value) {
        return (value & 0xff0000) >>> 16;
    }

    /**
     * Die Methode bestimmt HighByte.
     *
     * @param value Wert für den das HighByte bestimmt wird.
     *
     * @return HighByte Wert.
     */
    private int getHighByte(int value) {
        return (value & 0xff00) >>> 8;
    }

    /**
     * Die Methode bestimmt  LowByte.
     *
     * @param value Integervalue für den das LowByte bestimmt wird.
     *
     * @return lowByte.
     */
    private int getLowByte(int value) {
        return value & 0xff;
    }

    //~ METHODEN ==============================================================

    /**
     * Invertieren der Binaerdarstellung des Wertes.
     *
     * @param n   Wert der invertiert werden soll.
     * @param pos An welcher Position die invertierung durchgeführt werden soll.
     *
     * @return int mit invertiertem Bit des Wertes n, an Position pos.
     */
    int invertiereBit(int n, int pos) {
        return n & ~(1 << pos);
    }

    /**
     * Die Methode wertet einen Datensatz der Achslastmessstelle aus.
     *
     * @param rsArrayAtTime ResultData[] Archivanfrageresultdata.
     * @param calendar      Calendarobjekt zur Bestimmung der Zeit.
     *
     * @throws ChannelException Exception.
     */
    public void processRecord(Dataset[] rsArrayAtTime, Calendar calendar) throws ChannelException {
        Dataset dataSet = rsArrayAtTime[0];
        Data    data    = dataSet.getData();
        boolean istAxl;

        // deAxl:
        if ((data.getName().equals(BAStExportSteuerung.ATG_tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion3)) ||  //
            (data.getName().equals(BAStExportSteuerung.ATG_tlsAxlEinzelErgebnisMeldungAchsLastenUndKfzGesamtGewichteVersion1Bis2))) {
            istAxl       = true;
            _arIntValues = new int[MINLENGTH];
        }

        // deLve:
        // (data.getName().equals("atg.tlsLveKfzEinzelDaten")){
        else {
            istAxl       = false;
            _arIntValues = new int[LENGTH_LVE];
        }

        if (rsArrayAtTime.length > 0) {
            _arIntValues[1] = 1;
            _arIntValues[2] = bestimmeFahrstreifenkennung(rsArrayAtTime);
            bestimmeZeitangabe(calendar);

            // deAxl:
            if (istAxl) {
                int resizeGroesse = bestimmeTelegrammDatenFest(data);

                // Anpassen der Arraygroesse basierend auf der bestimmten Anzahl der Achslasten.
                reziseByteArray(resizeGroesse);

                for (int i = 0; i < resizeGroesse; i++) {
                    bestimmeTelegrammDatenvariabel(i, data);
                }
            }

            // deLve:
            else {
                bestimmeTelegrammDatenFestLve(data);
            }

            // Nachdem das Array vollstaendig vorbelegt ist, wird die Groesse in das erste Byte geschrieben.
            // Die Groesse wird dabei um 1 verkleinert (Längenbyte wird laut Formatvorgabe bei der Länge nicht mehr mitgezählt)
            _arIntValues[0] = _arIntValues.length - 1;

            // Schreibt die Daten in den Puffer, bis alle Datensäze eines Tages  ermittelt wurden.
            for (int aBinary : _arIntValues) {
                this.spool(toByte(aBinary));
            }
        }
    }

    /**
     * Die Methode vergroessert das Array um die uebergebene Groesse * 5, da pro Ueberlastungsachsen Datensatz 5 Werte
     * bestimmt werden.
     *
     * @param resizeGroesse Groesse um die das Array (multipliziert mit 5) vergroeßert wird.
     */
    private void reziseByteArray(int resizeGroesse) {
        _copyArIntValues = new int[_arIntValues.length];

        for (int i = 0; i < _copyArIntValues.length; i++) {
            _copyArIntValues[i] = _arIntValues[i];
        }

        _arIntValues = new int[(_arIntValues.length) + 5 * resizeGroesse];

        for (int i = 0; i < _copyArIntValues.length; i++) {
            _arIntValues[i] = _copyArIntValues[i];
        }
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setzen eines Bit an der Postion pos.
     *
     * @param n   Bit das Gesetzt wird.
     * @param pos Position an der das Bit gesetzt wird.
     *
     * @return Bit.
     */
    int setBit(int n, int pos) {
        return n | (1 << pos);
    }

    @Override
    public void setZaehlstelle(LangZeitZaehlStelle zaehlstelle) throws ChannelException {
        _ZaehlStelle = zaehlstelle;

        if (_Richtung == IN_RICHTUNG) {
            _AuswerteQuerschnitt = _ZaehlStelle.getInAuswertequerschnitt();
            _Fernziel            = _ZaehlStelle.getInFernziel();
        } else {
            _AuswerteQuerschnitt = _ZaehlStelle.getGegenAuswertequerschnitt();
            _Fernziel            = _ZaehlStelle.getGegenFernziel();
        }

        _FSArray = _AuswerteQuerschnitt.getMessQuerschnitt().getFahrstreifen();

        ArrayList<Fahrstreifen> list = new ArrayList<Fahrstreifen>();

        for (Fahrstreifen a_FSArray : _FSArray) {
            try {
                if (a_FSArray.isAchslastErfassung()) {
                    list.add(a_FSArray);
                }
            }
            catch (Exception e) {
                throw new ChannelException(e);
            }
        }

        _FSMitDatenerfassungArray = (Fahrstreifen[]) list.toArray(new Fahrstreifen[list.size()]);
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode toByte konvertiert den uebergebenen Integerwert in einen Bytewert.
     *
     * @param intValueOfByte Wert der konvertiert wird.
     *
     * @return konvertierte Bytewert.
     */
    private byte toByte(int intValueOfByte) {
        if ((intValueOfByte < 0) || (intValueOfByte > 255)) {
            throw new IllegalArgumentException("Wert liegt nicht zwischen 0 und 255 sondern ist " + intValueOfByte);
        }

        return (byte) intValueOfByte;
    }

    @Override
    protected void writeHeader(PrintWriter writer) throws IOException {

        // To change body of implemented methods use File | Settings | File Templates.
    }

    @Override
    protected void writeHeaderFTP(BufferedOutputStream bos) throws IOException {

        // To change body of implemented methods use File | Settings | File Templates.
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 20.01.15
