/*
 * Copyright 2008 by inovat, Dipl.-Ing. H. C. Kniß
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.kex.bast.importer;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.kex.bast.serverfunc.FTPFunktionen;
import org.apache.commons.net.ftp.FTPClient;

import java.io.InputStream;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse realisiert die FTP-Server-Kommunikation beim BASt-Band-Import
 * (s. {@link de.inovat.kex.bast.importer.BAStImportSteuerung#run()}).
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Liliya Givorgizova (LG)
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class FTPServerKommunikation implements IServerKommunikation {

    /** FTP-Benutzername. */
    private String _ftpBenutzer;

    /** FTPClient. */
    private FTPClient _ftpClient;

    /** FTP-Ordner, wo die zu bearbeitenden Dateien liegen. */
    private String _ftpOrdner;

    /** Legt fest, ob die FTP-Verbindung im PassivModus eingebaut werden soll. */
    private boolean _ftpPassivModus;

    /** FTP-Passwort. */
    private String _ftpPasswd;

    /** Host des FTP-Servers. */
    private String _ftpServerName;

    /** Port des FTP-Servers. */
    private int _ftpServerPort;

    //~ METHODEN ==============================================================

    @Override
    public InputStream ermittleDateiInputStream(String dateiPfad) {
        try {
            return _ftpClient.retrieveFileStream(dateiPfad);
        }
        catch (Exception e) {
            return null;
        }
    }

    @Override
    public void parameterUebergabe(String tempOrdner, String host, int port, String benutzer, String passwort, boolean passivModus, String ordner) {
        _ftpServerName  = host;
        _ftpServerPort  = port;
        _ftpBenutzer    = benutzer;
        _ftpPasswd      = passwort;
        _ftpPassivModus = passivModus;
        _ftpOrdner      = ordner;
    }

    @Override
    public Exception sperreDatei(String dateiMitPfad) {

        // Falls die Verbindung verloren geht, verbinde erneut:
        if (!_ftpClient.isConnected()) {
            Exception e = verbindeMitServer();

            if (e != null) {
                return e;
            }
        }

        // Suche die angegebene Datei auf dem Server:
        try {
            String[] alleFiles = _ftpClient.listNames(_ftpOrdner);

            for (String name : alleFiles) {

                // Sperre die Datei:
                if (name.equals(dateiMitPfad)) {
                    boolean renameOK = _ftpClient.rename(dateiMitPfad, String.format("%s%s", dateiMitPfad, BAStImportSteuerung.DATEI_LOCK));

                    if (renameOK) {
                        return null;
                    } else {
                        return new Exception(String.format("Die Datei <%s> konnte nicht gesperrt werden.%n", dateiMitPfad));
                    }
                }

                // Lock-Datei vorhanden:
                else if (name.equals(String.format("%s%s", dateiMitPfad, BAStImportSteuerung.DATEI_LOCK))) {
                    return new Exception(String.format(  //
                        "Die Datei <%s> konnte nicht gesperrt werden, da die Datei schon gesperrt ist!", dateiMitPfad));
                }

                // Bearbeitungs-Datei vorhanden:
                else if (name.equals(String.format("%s%s", dateiMitPfad, BAStImportSteuerung.DATEI_BEARBEITET))) {
                    return new Exception(String.format("Die Datei <%s> ist schon importiert worden!", dateiMitPfad));
                }
            }
        }
        catch (Exception e) {
            return new Exception("Die FTP-Verzeichnisinformationsdaten können nicht angezeigt werden.", e);
        }

        return new Exception(String.format(  //
            "Die angegebene Datei <%s> befindet sich nicht in dem Importverzeichnis! Bitte überprüfen Sie ihre Angaben!", dateiMitPfad));
    }

    @Override
    public Exception sperreDateiAufheben(String dateiMitPfad) {

        // Falls die Verbindung verloren geht, verbinde erneut:
        if (!_ftpClient.isConnected()) {
            Exception e = verbindeMitServer();

            if (e != null) {
                return e;
            }
        }

        // Umbenennen der Datei:
        try {
            String  dateiAlt = String.format("%s%s", dateiMitPfad, BAStImportSteuerung.DATEI_LOCK);
            String  dateiNeu = String.format("%s%s", dateiMitPfad, BAStImportSteuerung.DATEI_BEARBEITET);
            boolean renameOK = _ftpClient.rename(dateiAlt, dateiNeu);

            if (renameOK) {
                return null;
            } else {

                // 2.Versuch:
                Exception e = verbindeMitServer();

                if (e != null) {
                    return e;
                }

                renameOK = _ftpClient.rename(dateiAlt, dateiNeu);

                if (renameOK) {
                    return null;
                } else {
                    return new Exception(String.format("Die Sperre der Datei <%s> kann nicht aufgehoben werden.", dateiMitPfad));
                }
            }
        }
        catch (Exception e) {
            return new Exception(String.format("Die Sperre der Datei <%s> kann nicht aufgehoben werden.", dateiMitPfad), e);
        }
    }

    @Override
    public Exception trenneVerbindungMitServer() {
        try {
            _ftpClient.disconnect();

            return null;
        }
        catch (Exception e) {
            return e;
        }
    }

    @Override
    public Exception verbindeMitServer() {

        // Neue Verbindung:
        if (_ftpClient == null) {
            _ftpClient = new FTPClient();
        }

        // Disconnnect:
        else {
            try {
                _ftpClient.disconnect();
            }
            catch (Exception ignored) {}
        }

        // Verbindung:
        Exception exception = FTPFunktionen.verbindeMitFTPServer(_ftpClient, _ftpServerName, _ftpServerPort, _ftpBenutzer, _ftpPasswd);

        if (exception == null) {

            // Passiv/Aktiv-Modus:
            if (_ftpPassivModus) {
                _ftpClient.enterLocalPassiveMode();
            } else {
                _ftpClient.enterLocalActiveMode();
            }

            // OK:
            return null;
        } else {

            // _Fehler:
            return exception;
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 27.06.13
